log("custom_context_menu.js chargé correctement");

// === Variables globales ===
const WHITE_BOX_ID = "whiteBox";

// Fonction utilitaire pour envoyer une notification via le background
function sendNotification(title, message, iconPath) {
  browser.runtime.sendMessage({
    action: "showNotification",
    title,
    message,
    iconPath
  });
}

// Récupère le token depuis le stockage local et le stocke dans authToken
async function loadAuthToken() {
  try {
    const result = await browser.storage.local.get("accessToken");
    authToken = result.accessToken || null;
    log("🔑 Token chargé :", authToken);
  } catch (error) {
    log("❌ Erreur lors de la récupération du token :", error);
    authToken = null;
  }
}

/**
 * Crée le menu contextuel personnalisé (whiteBox) s'il n'existe pas déjà.
 */
function injectWhiteBox() {
  let whiteBox = document.getElementById(WHITE_BOX_ID);
  if (!whiteBox) {
    whiteBox = document.createElement("div");
    whiteBox.id = WHITE_BOX_ID;
    whiteBox.style.position = "absolute";
    whiteBox.style.zIndex = "9999";
    whiteBox.style.backgroundColor = "#fff";
    whiteBox.style.border = "1px solid #ccc";
    whiteBox.style.padding = "5px";
    whiteBox.style.borderRadius = "4px";
    whiteBox.style.boxShadow = "0px 2px 10px rgba(0,0,0,0.2)";
    const addLexiconPath = browser.runtime.getURL("src/assets/icons/ajout_lexique.png");
    const getDefinitionPath = browser.runtime.getURL("src/assets/icons/definition.png");
    const loginPath = browser.runtime.getURL("src/assets/icons/connexion.png");
    // Construction du HTML du menu contextuel
    whiteBox.innerHTML = `
      <p id="selectedWord" style="margin: 0; padding: 0;">Mot sélectionné : Aucun</p>
      <hr style="border: 0; height: 1px; background-color: #323046; margin: 8px 0;">
      <div style="display: flex; flex-wrap: wrap; justify-content: center;">
        <!-- Bouton : Ajouter au lexique -->
        <div class="icon-container" title="Ajouter ce mot à un lexique">
          <img src="${addLexiconPath}" alt="Ajouter au lexique" class="icon" id="addLexiconButton">
        </div>
        <!-- Bouton : Obtenir une définition -->
        <div class="icon-container" title="Obtenir la définition">
          <img src="${getDefinitionPath}" alt="Obtenir la définition" class="icon" id="getDefinitionButton">
        </div>
        <!-- Bouton : Connexion -->
        <div class="icon-container" title="Connectez-vous à BaLex">
          <img src="${loginPath}" alt="Se connecter" class="icon" id="loginButton" style="display: none;">
        </div>
      </div>
    `;
    document.body.appendChild(whiteBox);
    setupWhiteBoxActions();
  }
  whiteBox.addEventListener("mouseup", (e) => {
    e.stopPropagation();
  });
  return whiteBox;
}

/**
 * Renvoie le whiteBox s'il existe, ou le crée.
 */
function getOrCreateWhiteBox() {
  return document.getElementById(WHITE_BOX_ID) || injectWhiteBox();
}

/**
 * Configure les actions des boutons du menu contextuel.
 */
function setupWhiteBoxActions() {
  const addLexiconBtn = document.getElementById("addLexiconButton");
  const getDefinitionBtn = document.getElementById("getDefinitionButton");
  const loginBtn = document.getElementById("loginButton");

  // Tooltips
  if (loginBtn) {
    loginBtn.title = "En vous connectant, vous pourrez accéder à vos lexiques personnels ainsi qu'aux fonctionnalités d'ajout automatique et de statistiques d'utilisation.";
  }
  // Bouton : Ajouter le mot au lexique
  addLexiconBtn.onclick = async (e) => {
    e.stopPropagation();
    e.preventDefault();
    const selectedText = getSelectedWord().trim();
    log("🔍 Bouton Ajouter au lexique cliqué avec le mot :", selectedText);
    if (!selectedText) return;
    if (authToken) {
      browser.runtime.sendMessage({ action: "openLexiconBlock" });
      showPicker(e, selectedText);
    } else {
      alert("Vous devez être connecté pour ajouter un mot.");
    }
  };

  // Bouton : Obtenir une définition
  getDefinitionBtn.onclick = () => {
    const selectedText = getSelectedWord().trim();
    if (selectedText) {
      browser.runtime.sendMessage({
        action: "getDefinition",
        selectedText
      });
    }
  };

  // Bouton : Connexion
  loginBtn.onclick = () => {
    browser.runtime.sendMessage({ action: "toggleAuth" });
  };
}

/**
 * Met à jour la visibilité des boutons du menu selon l'authentification.
 */
function updateMenuVisibility() {
  getOrCreateWhiteBox();
  const addLexiconBtn = document.getElementById("addLexiconButton");
  const getDefinitionBtn = document.getElementById("getDefinitionButton");
  const loginBtn = document.getElementById("loginButton");

  if (!addLexiconBtn || !getDefinitionBtn || !loginBtn) {
    console.warn("⚠️ Un des boutons n'a pas été trouvé dans le DOM.");
    return;
  }

  if (authToken) {
    addLexiconBtn.style.display = "inline-block";
    getDefinitionBtn.style.display = "inline-block";
    loginBtn.style.display = "none";
  } else {
    addLexiconBtn.style.display = "none";
    getDefinitionBtn.style.display = "inline-block";
    loginBtn.style.display = "inline-block";
  }
}

/**
 * Récupère le texte affiché dans #selectedWord.
 */
function getSelectedWord() {
  const selectedWordElement = document.getElementById("selectedWord");
  return selectedWordElement ? selectedWordElement.textContent.trim() : "";
}

/**
 * Affiche le menu contextuel à la position du clic.
 */
function showWhiteBox(event, selectedText) {
  const whiteBox = getOrCreateWhiteBox();
  const selectedWordElement = document.getElementById("selectedWord");
  selectedWordElement.textContent = selectedText;

  const selection = window.getSelection();
  if (!selection.rangeCount) return;
  const range = selection.getRangeAt(0);
  const rect = range.getBoundingClientRect();
  const top = rect.bottom + window.scrollY;
  const left = rect.right + window.scrollX;

  whiteBox.style.left = left + "px";
  whiteBox.style.top = top + "px";
  whiteBox.style.display = "block";

  log("Affichage du menu contextuel avec le mot :", selectedText);
  updateMenuVisibility();
}

function hideWhiteBox() {
  const whiteBox = document.getElementById(WHITE_BOX_ID);
  if (whiteBox) {
    whiteBox.style.display = "none";
  }
}

// Écoute globale pour la sélection de texte
document.addEventListener("mouseup", (event) => {
  if (event.target.closest("#whiteBox")) return;
  const selectedText = window.getSelection().toString().trim();
  if (selectedText) {
    log("Texte sélectionné :", selectedText);
    getOrCreateWhiteBox();
    showWhiteBox(event, selectedText);
    browser.runtime.sendMessage({
      action: "mot_selectionne",
      selectedText,
    });
  } else {
    hideWhiteBox();
  }
});

// Écoute des messages entrants
browser.runtime.onMessage.addListener((message) => {
  if (message.action === "refreshUI") {
    log("🔄 Mise à jour du menu contextuel personnalisé.");
    loadAuthToken().then(updateMenuVisibility);
  }
});

// Initialisation au démarrage
loadAuthToken().then(() => {
  getOrCreateWhiteBox();
  updateMenuVisibility();
});
browser.storage.onChanged.addListener((changes) => {
  if (changes.accessToken) {
    log("🔄 Token modifié dans le stockage, mise à jour du menu contextuel.");
    loadAuthToken().then(updateMenuVisibility);
  }
});

// ─────────────────────────────────────────────────────────────────────────────
// Fonctions d'API pour l'ajout d'un mot via le sélecteur
// ─────────────────────────────────────────────────────────────────────────────
/**
 * Affiche le sélecteur pour choisir le lexique dans lequel ajouter le mot.
 */
async function showPicker(event, selectedText) {
  let picker = document.getElementById("lexiconPicker");
  if (!picker) {
    picker = document.createElement("div");
    picker.id = "lexiconPicker";
    picker.addEventListener("mouseup", (e) => e.stopPropagation());
    document.body.appendChild(picker);
  }
  picker.innerHTML = "";
  const selectedLexicons = new Set();

  try {
    const lexicons = await getLexicons(authToken);
    log("Lexicons récupérés :", lexicons);

    const lexiconDescriptions = {};

    if (!Array.isArray(lexicons) || lexicons.length === 0) {
      picker.innerHTML = "<p style='color:#333;'>Aucun lexique trouvé.</p>";
    } else {
      for (const lex of lexicons) {
        const id = lex.id;
        let name = "";
        if (lex.category === "User") {
          name = DEBUG
            ? `Lexique personnel : ${lex.user?.pseudo || "Inconnu"} (${lex.id})`
            : `Lexique personnel : ${lex.user?.pseudo || "Inconnu"}`;
        } else {
          name = DEBUG
            ? `Lexique de groupe : ${lex.group?.name || "Inconnu"} (${lex.id})`
            : `Lexique de groupe : ${lex.group?.name || "Inconnu"}`;
        }
        if (lex.language) {
          name += ` [${lex.language}]`;
        }
        lexiconDescriptions[id] = name;
        const color = await getColorForLexicon(id);
        const circleIcon = await createColorCircle(color, 28);
        
        const iconContainer = document.createElement("div");
        iconContainer.className = "lexicon-option";
        iconContainer.dataset.lexiconId = id;
        iconContainer.title = name;
        iconContainer.addEventListener("click", () => {
          if (selectedLexicons.has(id)) {
            selectedLexicons.delete(id);
            iconContainer.classList.remove("selected");
          } else {
            selectedLexicons.add(id);
            iconContainer.classList.add("selected");
          }
        });
        iconContainer.appendChild(circleIcon);
        picker.appendChild(iconContainer);
      }

      const confirmButton = document.createElement("button");
      confirmButton.className = "confirmButton";
      confirmButton.textContent = "Ajouter le mot";
      confirmButton.addEventListener("click", async () => {
        if (selectedLexicons.size === 0) {
          alert("Veuillez sélectionner au moins un lexique.");
          return;
        }
        log(`🔍 Vérification si le mot "${selectedText}" existe déjà dans les lexiques sélectionnés...`);
        let definitions = [];
        try {
          definitions = await fetchLexiconDefinitions(selectedText);
        } catch (error) {
          log("Erreur lors de la récupération des définitions :", error);
        }
        const existingLexiconIds = new Set();
        if (Array.isArray(definitions)) {
          for (const def of definitions) {
            if (selectedLexicons.has(def.lexiconId)) {
              existingLexiconIds.add(def.lexiconId);
            }
          }
        }
        if (existingLexiconIds.size > 0) {
          alert(`⚠️ Le mot "${selectedText}" existe déjà dans les lexiques suivants : ${Array.from(existingLexiconIds).join(", ")}`);
        }
        const lexiconsToAdd = [...selectedLexicons].filter(id => !existingLexiconIds.has(id));
        if (lexiconsToAdd.length === 0) {
          alert(`✅ Le mot "${selectedText}" est déjà présent dans tous les lexiques sélectionnés.`);
          return;
        }
        try {
          log(`📡 Ajout du mot "${selectedText}" dans les lexiques :`, lexiconsToAdd);
          const result = await AddWord(authToken, selectedText, lexiconsToAdd, false);
          log("Réponse API :", result);
          await new Promise(resolve => setTimeout(resolve, 300));
          browser.runtime.sendMessage({ action: "refreshUI" });
          const successMsg = `✅ Mot ajouté avec succès dans : ${lexiconsToAdd.map(id => lexiconDescriptions[id]).join(", ")}`;
          picker.innerHTML = `<p style="color: green;">${successMsg}</p>`;
          setTimeout(() => picker.style.display = "none", 2000);
          browser.runtime.sendMessage({
            action: "addWordResult",
            lexicons: successMsg
          });
        } catch (error) {
          log("❌ Erreur lors de l'ajout du mot :", error);
          const errorMsg = `❌ Erreur lors de l'ajout du mot : ${error.message}`;
          picker.innerHTML = `<p style="color: red;">${errorMsg}</p>`;
          setTimeout(() => picker.style.display = "none", 3000);
          browser.runtime.sendMessage({
            action: "addWordResult",
            lexicons: errorMsg
          });
        }
      });
      picker.appendChild(confirmButton);
    }

    const lexiconCount = Array.isArray(lexicons) ? lexicons.length : 0;
    const baseWidthPerLexicon = 40; 
    const extraPadding = 20;
    const calculatedWidth = lexiconCount * baseWidthPerLexicon + extraPadding;
    picker.style.width = calculatedWidth + "px";
    picker.style.left = event.pageX + "px";
    picker.style.top = event.pageY + "px";
    picker.style.display = "flex";
  } catch (error) {
    log("❌ Erreur lors de la récupération des lexiques :", error);
    picker.innerHTML = "<p style='color:#333;'>Erreur lors du chargement des lexiques.</p>";
    picker.style.display = "block";
  }
}

function hideLexiconPicker() {
  const picker = document.getElementById("lexiconPicker");
  if (picker) {
    picker.style.display = "none";
  }
}

document.addEventListener("mouseup", (event) => {
  const whiteBox = document.getElementById(WHITE_BOX_ID);
  const picker = document.getElementById("lexiconPicker");

  if (whiteBox && !whiteBox.contains(event.target)) {
    hideWhiteBox();
  }
  if (picker && !picker.contains(event.target)) {
    hideLexiconPicker();
  }
  
  const selectedText = window.getSelection().toString().trim();
  if (selectedText) {
    log("Texte sélectionné :", selectedText);
    showWhiteBox(event, selectedText);
    browser.runtime.sendMessage({
      action: "mot_selectionne",
      selectedText,
    });
  }
});
